/***************************************************************************
                    lmsensordockpanel.cpp  -  description
                             -------------------
    begin                : Tue Sep 18 2001
    Copyright	Giuseppe Della Bianca<giusdbg@gmail.com>

    original author	 : Miguel Novas
    email                : michaell@teleline.es
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <cmath>

#include <qdebug.h>
#include <qevent.h>
#include <qpixmap.h>
#include <qpixmapcache.h>
#include <qapplication.h>

#include <kaboutdata.h>
#include <klocalizedstring.h>

#include "general.h"
#include "sensorscfg.h"
#include "lmsensordockpanel.h"


static QColor color2( 200, 0, 0);
static QColor color3( 128,32,0 );

LMSensorDockPanel::LMSensorDockPanel(Sensor *newSensor, KsensorsMain *lmsdKSensors, const QString &name) : KStatusNotifierItem ( name, /* newSensor */ nullptr ) /* : KSystemTrayIcon() */
{
  setObjectName ( name );
  
  qpIcon = nullptr;
  barMeter = nullptr;

  kmContext = nullptr;
  qaMinimizeRestore = nullptr;
  
  strValue.clear();
 
  const KAboutData kadAboutData = KAboutData::applicationData();
  
  QString qsDockImage = findIcon ("ksensorsdocked.png");
  QPixmap qpDockImage = QPixmap ( qsDockImage );
  
//  setIconByName( qsDockImage );
  setToolTipIconByPixmap( qpDockImage );
  setIconByPixmap( qpDockImage );

//  setToolTipIconByName( kadAboutData.displayName() );    
  if (newSensor)
    setTitle( kadAboutData.displayName() + qsFL1( " - " ) + newSensor->getDescription() + qsFL1( " (" ) + newSensor->getLongName() + qsFL1( ")" ) );
  else
  {
    setToolTipTitle( kadAboutData.displayName() );
    setToolTipSubTitle( qsFL1( "Left clik for open/close panels window." ) + QLatin1String(  "<br>"  ) + qsFL1( "Right click for Configure and Save Configuration and other." ) );
  }
  
  setStatus( KStatusNotifierItem::Active );
  setCategory( Hardware );
  setStandardActionsEnabled( false );
 
 if(newSensor) 
 {
    sensor= newSensor;

    slotUpdateConfig();

    connect( newSensor, SIGNAL(valueChanged()) , this, SLOT(setValue()) );
    connect( newSensor, SIGNAL(panelDockConfigChanged()), this, SLOT(slotUpdateConfig()) );
    
 } 
 else 
 {
    sensor= nullptr;
 }
 
 
 lmsdKSensors->createMenu ( kmContext, qaMinimizeRestore );
 
 setContextMenu( kmContext );
 
    connect( this, SIGNAL (activateRequested(bool,const QPoint)), this, SLOT (trayIconActivated(bool,const QPoint)) );
}

LMSensorDockPanel::~LMSensorDockPanel()
{
}

void LMSensorDockPanel::setToolTipValue ( QString value )
{
  const KAboutData kadAboutData = KAboutData::applicationData();
  
  setToolTipTitle( kadAboutData.displayName() + qsFL1( " - " ) + sensor->getDescription() + qsFL1( " (" ) + sensor->getLongName() + qsFL1( ")" ) );
  setToolTipSubTitle( value );
}

void LMSensorDockPanel::setValue( /* double value, double value1, double value2, double value3, double value4 */ )
{
  QString strToolTip;
  
  double value = sensor->getValue();
  double value1 = sensor->getValue1();
  double value2 = sensor->getValue2();
  double value3 = sensor->getValue3();
  double value4 = sensor->getValue4();
  QString valDescr = sensor->getValDescr();
  QString descr1 = sensor->getDescr1();
  QString descr2 = sensor->getDescr2();

  struct Sensor::stValMask vMaskV;
  
  strToolTip.clear();
  
  if ( sensor ) 
  {     
    vMaskV = sensor->getPrintMask( true );
    
    switch( sensor->getType() ) 
    {
    case Sensor::lmTemp:
      strValue = qsFL1( "%1" ).arg ( value, 2, 'f', 0, QChar::fromLatin1( ' ' ) );
      
      strToolTip = qsFL1( "TEMP %1%2%3" ).arg( vMaskV.preTxt ).arg ( value, vMaskV.width, vMaskV.formType, vMaskV.prec, QChar::fromLatin1( ' ' ) ).arg ( vMaskV.postTxt );
                  
      break;
    case Sensor::lmFan:
      strValue = qsFL1( "%1" ).arg ( value / 1000, 1, 'f', 1, QChar::fromLatin1( ' ' ) );      

      strToolTip = qsFL1( "SPEED %1%2%3" ).arg( vMaskV.preTxt ).arg ( value, vMaskV.width, vMaskV.formType, vMaskV.prec, QChar::fromLatin1 ( ' ' ) ).arg ( vMaskV.postTxt );
      
      break;
    case Sensor::lmVoltage:
      strValue = qsFL1( "%1" ).arg ( value, 2, 'f', 1, QChar::fromLatin1( ' ' ) );
      
      strToolTip = qsFL1( "VOLT %1%2%3" ).arg( vMaskV.preTxt ).arg ( value, vMaskV.width, vMaskV.formType, vMaskV.prec, QChar::fromLatin1( ' ' ) ).arg ( vMaskV.postTxt );
      
      break;
    case Sensor::sysCpuInfo:
      strValue = qsFL1( "%1" ).arg ( value );

      if ( barMeter )
      {
        barMeter->setValue( 0, value );
        barMeter->setValueMax( value, true );
      }
      
      strToolTip = qsFL1( "CPU %1" ).arg ( descr1 );
      strToolTip += QLatin1String(  "<br>"  )+ qsFL1( "SPEED %1" ).arg ( value , 5, 'f', 0, QChar::fromLatin1( ' ' ) );
      strToolTip += QLatin1String(  "<br>"  )+ qsFL1( "BOGOMIPS %1" ).arg ( descr2 );
      
      break;
    case Sensor::sysCpuLoad:
      strValue = qsFL1( "%1" ).arg ( value, 2, 'f', 0, QChar::fromLatin1( ' ' ) );

      if ( barMeter )
      {
        barMeter->setValue( 0, value1 );
        barMeter->setValue( 1, value2 );
        barMeter->setValue( 2, value3 );
        barMeter->setValue( 3, value4 );
      }

      strToolTip = qsFL1( "IDLE%1" ).arg ( value4, 5, 'f', 1, QChar::fromLatin1 ( ' ' ) );
      strToolTip += QLatin1String(  "<br>"  )+ qsFL1( "SYS %1" ).arg ( value3 , 5, 'f', 1, QChar::fromLatin1( ' ' ) );
      strToolTip += QLatin1String(  "<br>"  )+ qsFL1( "NICE%1" ).arg ( value2 , 5, 'f', 1, QChar::fromLatin1( ' ' ) );
      strToolTip += QLatin1String(  "<br>"  )+ qsFL1( "USER%1" ).arg ( value1 , 5, 'f', 1, QChar::fromLatin1( ' ' ) );

      break;
   case Sensor::sysRamUsed:
   case Sensor::sysSwapUsed:
      strValue = qsFL1( "%1" ).arg ( value );

      if ( barMeter )
      {
        barMeter->setValue( 0, value );
        barMeter->setValueMax( value1, false );
      }

      strToolTip = qsFL1( "AVAILABLE %1 M" ).arg ( value1, 8, 'f', 0, QChar::fromLatin1( ' ' ) );
      strToolTip += QLatin1String(  "<br>"  )+ qsFL1( "USED %1 M" ).arg ( value , 8, 'f', 0, QChar::fromLatin1( ' ' ) );

      break;
   case Sensor::sysUpTime:
      strValue = valDescr;

      strToolTip = qsFL1( "UPTIME %1 HH:MM" ).arg ( valDescr );
      
     break;
   }

   if ( ! strToolTip.isEmpty() )
      setToolTipValue ( strToolTip );
   
   paintEvent( nullptr );
 }
}

void LMSensorDockPanel::updateMenu( QString qsActionMinRest )
{
  qaMinimizeRestore->setText( qsActionMinRest );
}

void LMSensorDockPanel::slotUpdateConfig()
{
    if ( qsPixMapSize.isEmpty() )
    {

  QSize qsPixMapAct = iconPixmap().actualSize ( QSize (100, 100) );
        
  int preferredItemSize = 128;
  int baseSize = qsPixMapAct.height();
  int suggestedsize = fmin(baseSize * 2, preferredItemSize);
  
  if (suggestedsize < 16 || suggestedsize >= 64) {
      qsPixMapSize = QSize(suggestedsize, suggestedsize);
  } else if (suggestedsize < 22) {
      qsPixMapSize = QSize(16, 16);
  } else if (suggestedsize < 24) {
      qsPixMapSize = QSize(22, 22);
  } else if (suggestedsize < 32) {
      qsPixMapSize = QSize(24, 24);
  } else if (suggestedsize < 48) {
      qsPixMapSize = QSize(32, 32);
  } else if (suggestedsize < 64) {
      qsPixMapSize = QSize(48, 48);
  }

    }

  if ( qpIcon )
  {
    delete qpIcon;
    qpIcon = nullptr;
  }
  if ( barMeter )
  {
    delete barMeter;
    barMeter = nullptr;
  }

  switch( sensor->getType() )
  {
  case Sensor::sysCpuInfo:
    barMeter= new QBarMeter();
    barMeter->setGeometry( 0, 0, qsPixMapSize.width(), qsPixMapSize.height() );
    barMeter->setDirection( QBarMeter::dirUp );
    barMeter->setSteps( 15 );
    barMeter->useValueMax( true );
    barMeter->setValueColor( 0, Qt::darkRed );
    barMeter->setValueMaxColor( Qt::darkGray );
    barMeter->setValueCount( 1 );
    
    break;
  case Sensor::sysCpuLoad:
    barMeter= new QBarMeter(/*this */);
    barMeter->setGeometry( 0, 0, qsPixMapSize.width(), qsPixMapSize.height() );
    barMeter->setDirection( QBarMeter::dirUp );
    barMeter->setSteps( 15 );
    barMeter->useValueMax(false);
    barMeter->setValueColor( 0, Qt::red );
    barMeter->setValueColor( 1, color2 );
    barMeter->setValueColor( 2, color3 );
    barMeter->setValueColor( 3, QColor(0,220,0) );
    barMeter->setValueCount(4);

    break;
  case Sensor::sysRamUsed:
  case Sensor::sysSwapUsed:
    barMeter= new QBarMeter();
    barMeter->setGeometry( 0, 0, qsPixMapSize.width(), qsPixMapSize.height() );
    barMeter->setDirection( QBarMeter::dirRight );
    barMeter->setSteps( 15 );
    barMeter->useValueMax( true );
    barMeter->setValueColor( 0, Qt::darkRed );
    barMeter->setValueMaxColor( Qt::darkGray );
    barMeter->setValueCount( 1 );

    break;
  case Sensor::sysUpTime:
  default:

    break;
  } // switch( sensor->getType() )

  if ( !barMeter )
  {
    qpIcon = new QPixmap ( qsPixMapSize.width(), qsPixMapSize.height() );
    qpIcon->fill ( Qt::transparent );
  }

//  setGroupConf ( objectName() );
    setSensorDevGroupConf ( sensor->getShortName(), sensor->getLongName(), sensor->getSubName() );
    colorAlarm = getEntryConf ( qsFL1( "colorAlarm" ), QColor(), ksensorsSetting::AutoDefault );    
    colorNormal = getEntryConf ( qsFL1( "colorNormal" ), QColor(), ksensorsSetting::AutoDefault );
 
 setValue( /* sensor->getValue(), 0.0, 0.0, 0.0, 0.0  */ );
}

void LMSensorDockPanel::paintEvent ( QPaintEvent *e )
{
  Q_UNUSED(e);
  
 if(sensor) 
 {   
    int w = qsPixMapSize.width();
    int h = qsPixMapSize.height();

    QColor color= sensor->getAlarm() ? colorAlarm : colorNormal;
//    QColor colorShadow= color.dark(200);
    QColor colorShadow= color.lighter( 200 );
   

    if ( barMeter )
    {
      setIconByPixmap ( barMeter->grab( QRect ( 0, 0, w, h ) ) );
    }
    else if ( qpIcon )
    {
//      delete qpIcon;
      QPixmap pixmap ( *qpIcon );
      
      QPainter qpIconPainter( &pixmap );

/*    qpIconPainter.setPen(color);
      qpIconPainter.drawLine(2, 2  , w-2,2  );
      qpIconPainter.drawLine(2, h-3, w-2,h-3);
      qpIconPainter.setPen(colorShadow);
      qpIconPainter.drawLine(3, 3  , w-3,3  );
      qpIconPainter.drawLine(3, h-4, w-3,h-4); */
      if(sensor->getType()==Sensor::lmTemp) {
        QLcd::draw(&qpIconPainter, /* 4,6,w-6, h-12, */ 0, 0, w, h, strValue, QLcd::drawShadow,&color,&colorShadow);
      } else {
        QLcd::draw(&qpIconPainter, /* 1,7,w-1, h-14, */ 0, 0, w, h, strValue, QLcd::drawNumber|QLcd::alignCenter|QLcd::drawShadow,&color,&colorShadow);
      }

/*    if(!pixmap.save("/tmp/pippo.png", "png"))
          qDebug() << "Save icon failed!!"; */
//    setIconByName("/tmp/pippo.png");

      setIconByPixmap( pixmap );
    }
    
    QPixmapCache::clear();
       
 } 
 else 
 {   
//   KSystemTrayIcon::paintEvent(e);
     qDebug() << "\033[91m" << "qDebug(): LMSensorDockPanel::paintEvent()" << "//   KSystemTrayIcon::paintEvent(e); !!" << "\033[0m";   
 }
}

void LMSensorDockPanel::trayIconActivated( bool active, const QPoint &pos )
{
    Q_UNUSED(active);

#if (QT_VERSION < QT_VERSION_CHECK(6, 0, 0))  
    QMouseEvent e ( QEvent::MouseButtonRelease, pos, Qt::LeftButton, Qt::LeftButton, Qt::NoModifier );
#else  
    QMouseEvent e ( QEvent::MouseButtonRelease, pos.toPointF(), pos.toPointF(), Qt::LeftButton, Qt::LeftButton, Qt::NoModifier );
#endif
  
    Q_EMIT mouseEvent(&e);

//  qDebug() << "LMSensorDockPanel::activated()";
}

/* void LMSensorDockPanel::mousePressEvent ( QMouseEvent *e )
{
 emit mouseEvent(e);
}

void LMSensorDockPanel::mouseReleaseEvent ( QMouseEvent *e )
{
 emit mouseEvent(e);
} */
