/***************************************************************************
                          lmsensorsalarms.cpp  -  description
                             -------------------
    begin                : Wed Nov 14 2001
    Copyright	Giuseppe Della Bianca<giusdbg@gmail.com>

    original author	 : Miguel Novas
    email                : michaell@teleline.es
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <stdlib.h>

#include <qstring.h>
#include <qapplication.h>

#include <kprocess.h>
#include <kaboutdata.h>

#include "general.h"
#include "lmsensorsalarms.h"


// Phonon::MediaObject *LMSensorsAlarms::media = nullptr;
QMediaPlayer *LMSensorsAlarms::media = nullptr;

LMSensorsAlarms::LMSensorsAlarms( SensorDevsList *sensors, QObject *parent, const QString &name ) : QObject( /* parent,name */ )
{
   Q_UNUSED(parent);
   Q_UNUSED(name);
   
   idTimer = 0;
 sensorCount= 0;
 soundPlay.clear();
 connect( sensors, SIGNAL(valueChanged(Sensor *)) , this, SLOT(setValueChanged(Sensor *)) );

//    media = new Phonon::MediaObject( this );
//    Phonon::createPath( media, new Phonon::AudioOutput( Phonon::MusicCategory, this ));
    media = new QMediaPlayer();
#if (QT_VERSION < QT_VERSION_CHECK(6, 0, 0))
    media->setVolume(100);
#else
//    connect(QMediaPlayer, &QMediaPlayer::positionChanged, this, &MediaExample::positionChanged);
    QAudioOutput *qaoMedia = new QAudioOutput();
    media->setAudioOutput ( qaoMedia );
    qaoMedia->setVolume ( 100 );
#endif
}

LMSensorsAlarms::~LMSensorsAlarms()
{
}

//###########################################################################################


void LMSensorsAlarms::runSensorCommand( const QString &name, const QString &value, const QString &cmd)
{
  setenv("SENSOR_NAME" , name.toLatin1().data(), 1);
  setenv("SENSOR_VALUE", value.toLatin1().data(), 1);
  runCommand( cmd );
}

void LMSensorsAlarms::runCommand ( const QString &str )
{
   KProcess p;

   p << str;
   p.startDetached(/* KShellProcess::DontCare */);
}

void LMSensorsAlarms::playSound( const QString &sound )
{
    QString qsPreFile, qsSoundFile;
    
   const KAboutData kadAboutData = KAboutData::applicationData();
   
   qsPreFile = kadAboutData.componentName() + qsFL1( "/sounds/" ) + sound;
   
   qsSoundFile = QStandardPaths::locate( QStandardPaths::GenericDataLocation, qsPreFile );
   
   qCDebug(qlcLMSensorsAlarms) << "\033[91m" << "playSound " << QStandardPaths::GenericDataLocation << qsPreFile << "\033[0m";
   qCDebug(qlcLMSensorsAlarms) << "\033[91m" << "playSound " << sound << qsSoundFile << "\033[0m";
   
//    media->setCurrentSource( QUrl::fromLocalFile ( qsSoundFile ) );
#if (QT_VERSION < QT_VERSION_CHECK(6, 0, 0))
    media->setMedia( QUrl::fromLocalFile ( qsSoundFile ) );
#else
    media->setSource( QUrl::fromLocalFile ( qsSoundFile ) );
#endif
    media->play();
}

//###########################################################################################

void LMSensorsAlarms::setValueChanged(Sensor *sensor)
{
//  setGroupConf ( sensor->objectName() );
    setSensorDevGroupConf ( sensor->getShortName(), sensor->getLongName(), sensor->getSubName() );
   
 if(sensor->getAlarm()) {
    if(!sensor->getAlarmPrevious()) 
    {       
       switch ( getEntryConf ( qsFL1( "Alarm" ), -1, ksensorsSetting::AutoDefault ) )
       {
        case acSound:

           sensorCount++;
           soundPlay= getEntryConf  (qsFL1( "AlarmSound" ), QString(), ksensorsSetting::AutoDefault );
           if(sensorCount==1) 
           {
              if ( !idTimer )
                  idTimer = startTimer(3000);
           }
           
           break;

        case acCommand:

           runSensorCommand(
                              sensor->getDescription(),
                              QString::number(sensor->getValue()),
                              getEntryConf  ( qsFL1( "AlarmCommand" ), QString(), ksensorsSetting::AutoDefault )
                            );
           break;
       }
    }
 } else {
    if(sensor->getAlarmPrevious()) {
       if( getEntryConf ( qsFL1( "Alarm" ), -1, ksensorsSetting::AutoDefault ) ==acSound ) {
          if(--sensorCount==0)  
          {
             if ( idTimer )
             {
               killTimer( idTimer );
               idTimer = 0;
             }
          }
       }
    }
 }
}

void LMSensorsAlarms::timerEvent( QTimerEvent * )
{
//    media->setCurrentSource( QUrl::fromLocalFile (soundPlay) );
#if (QT_VERSION < QT_VERSION_CHECK(6, 0, 0))
    media->setMedia( QUrl::fromLocalFile ( soundPlay ) );
#else
    media->setSource( QUrl::fromLocalFile ( soundPlay ) );
#endif
    media->play();
}
