/***************************************************************************
                          panelswindow.cpp  -  description
                             -------------------
    begin                : Mon Aug 6 2001
    copyright            : (C) 2001 by 
    email                : 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qobject.h>
#include <qpixmap.h>
#include <qtimer.h>
#include <qevent.h>
#include <qicon.h>
#include <qapplication.h>
#include <qdebug.h>

#include <klocalizedstring.h>
#include <kaboutdata.h>

#include "general.h"
#include "sensorscfg.h"
#include "panelswindow.h"


/***************************************************************************
 *  Public methods                                                         *
 ***************************************************************************/

PanelsWindow::PanelsWindow ( KsensorsMain *ksensorsMainIn, MeterTypes meterTypesIn, QWidget *parent, const QString &name ) : QWidget( parent, Qt::WindowStaysOnTopHint /* ,name */)
{
    Q_UNUSED(name);
  
/*   WStyle_Customize | WStyle_StaysOnTop | WStyle_NormalBorder | */
//  setWindowFlags( Qt::Widget );
    setAttribute( Qt::WA_DeleteOnClose );  

    ksensorsMain = ksensorsMainIn;
    
    meterTypes = MeterTypes::None;

    qlMess = nullptr;

    meterWidth = meterHeight = 64;

    setMinimumSize(0,0);
/*  setSizeIncrement(64,64);
    setBaseSize(64,64); */

    meterTypes = meterTypesIn;

 //
 childDraging = nullptr;
 //
 const KAboutData kadAboutData = KAboutData::applicationData();
 
//    setWindowTitle( kadAboutData.displayName() );
    setWindowTitle( i18n( "Gauge") );
    if ( meterTypes & MeterTypes::Chart )
        setWindowTitle( i18n( "Chart") );
 
 //
 windowsSolid = true;
 qpmBackGround = QPixmap ( findIcon ("ksensorsbackground.png") );
 qpPanelWindow = palette();
 loadPalette();

//    setSizeIncrement( 64, 64 );
//    setBaseSize( 64, 64);
 //

    setGroupConf ( qsFL1( "ShowWidget" ) );
    QRect rect = getEntryConf ( qsFL1( "geometry" ), meterTypes, QRect(), ksensorsSetting::AutoDefault );

    move( rect.topLeft() );
    resize( rect.size() );

    loadGeneralOptions( true );

 // 
// setBackgroundPixmap( QPixmap( findIcon ("ksensorsbackground.png") ) ); 
 //
  
 //
 createSensorPanels();
 //
 
// make windowHandle() find struct QTLWExtra -> QWidgetWindow *window not set to zero
  WId wiForSetIncr = winId();
  Q_UNUSED(wiForSetIncr);
/*  qtResize.setSingleShot(true);
  connect(&qtResize, SIGNAL(timeout()), this, SLOT(resizeDone())); */

}

PanelsWindow::~PanelsWindow()
{  
  saveGeometry();
}

void PanelsWindow::updateWidget ( const QString &name, bool withChange )
{
    bool withShow;
    
    QString qsSensorShowType;

    MeterTypes sensorMeterTypes;
    
    if ( !name.isEmpty() && name != qsFL1( "General" ) )
    {
        Panel *panel= findChild<Panel *>(name);
        Sensor *sensor = ksensorsMain->sensorDevsList->getSensor(name);

        if( sensor )
        {
//          setGroupConf ( name );
            setSensorDevGroupConf ( sensor->getShortName(), sensor->getLongName(), sensor->getSubName() );

            qsSensorShowType = getEntryConf ( qsFL1( "show" ), qsFL1( "false" ), ksensorsSetting::AutoDefault );

            
            sensorMeterTypes = MeterTypes::None;
            if ( qsSensorShowType.indexOf ( qsFL1( "gauge" ) ) >= 0 )
                sensorMeterTypes = (MeterTypes)( sensorMeterTypes | MeterTypes::Gauge );
            if ( qsSensorShowType.indexOf ( qsFL1( "chart" ) ) >= 0 )
                sensorMeterTypes = (MeterTypes)( sensorMeterTypes | MeterTypes::Chart );

            
            withShow = sensorMeterTypes & meterTypes;

            if ( withShow != (bool)panel )
                withChange = true;

          if ( withChange ) 
          {
              if(panel) 
                  delete panel; 
            
              if ( withShow )
                (void)new SensorPanel( this, sensor, sensor->getLongName() );
          }
      } 

      panel= findChild<Panel *>(name);
    if(panel) loadPalette(panel);
    } 
    else 
    {
    loadPalette();
    loadGeneralOptions( false );
    }
    
    
    updateMessLabel ( true );
}

void PanelsWindow::updatePanels( bool enabForce )
{
    bool enabSize;
    int newMeterWidth, newMeterHeight;

    setGroupConf ( qsFL1( "ShowWidget" )  );
    newMeterWidth = getEntryConf ( qsFL1( "meterWidth" ), meterTypes, -1, ksensorsSetting::AutoDefault );
    newMeterHeight = getEntryConf ( qsFL1( "meterHeight" ), meterTypes, -1, ksensorsSetting::AutoDefault );

  const QObjectList *panels= &children();

  enabSize = false;

  if ( enabForce || meterWidth != newMeterWidth || meterHeight != newMeterHeight ) 
  {
      int sizeIncrWitdh, sizeIncrHeight;
      
      sizeIncrWitdh = ksensorsMain->gaugeIncrWidth; 
      sizeIncrHeight = ksensorsMain->gaugeIncrHeight;
      if ( meterTypes & MeterTypes::Chart )
      {
          sizeIncrWitdh = ksensorsMain->chartIncrWidth; 
          sizeIncrHeight = ksensorsMain->chartIncrHeight;
      }

      setSizeIncrement( sizeIncrWitdh, sizeIncrHeight );
      setBaseSize ( sizeIncrWitdh, sizeIncrHeight );

    enabSize = true;
  }
    
  if ( panels )
  {
    for ( QObjectList::ConstIterator it = panels->begin(); it != panels->end(); ++it )       
    {
//      Panel *panel = (Panel *)(*it);
      Panel *panel = qobject_cast<Panel *>( *it );
      QWidget *qwPanel = (QWidget *)(*it);
       
      if ( panel )
      {
      
      if ( qwPanel )
      {

        if ( enabSize )
        {
          qwPanel->move( (qwPanel->x()/meterWidth)*newMeterWidth, (qwPanel->y()/meterHeight)*newMeterHeight);
          
          /* qwPanel */ panel->resizePanel ( newMeterWidth, newMeterHeight );
        }
      
        loadPalette ( panel ) ;
      }
      else
        break;
        
      }
      
    }      
  } // if ( panels )

    if ( enabSize )
    {
        meterWidth = newMeterWidth;
        meterHeight = newMeterHeight;

        adjustSize();
    }
} // void updatePanels()

/***************************************************************************
    update value
 ***************************************************************************/

void PanelsWindow::updateValue( const QString & name )
{
    SensorPanel *sensorPanel;

    sensorPanel = findChild<SensorPanel *>(name);

    if ( sensorPanel )
        sensorPanel->updateValue();
}

/***************************************************************************
    update message label
 ***************************************************************************/

void PanelsWindow::updateMessLabel( bool enabView )
{
    bool newVisible;
  
    QRect qrWin, qrMess;

    if ( !qlMess )
    {
        qlMess = new QLabel ( this );
        qlMess->setDisabled ( true );
        qlMess->setMargin( 5 );
        qlMess->setWordWrap ( true );
        qlMess->setText( i18n( "In this panels window or in panels docked to the system tray" ) + QLatin1String( "<br>" ) + i18n( "Right click for Configure and Save Configuration and other." ) + QLatin1String( "<br>" ) + QLatin1String( "<br>" ) + i18n( "Left clik on panels docked to the system tray for open/close panels window." ) );
        
        qlMess->setVisible ( false );        
    }


    newVisible = enabView && children().size() <= 1;

    if ( newVisible != qlMess->isVisible() )
    {
        if ( newVisible )
        {
            qlMess->setVisible ( true );

            qlMess->adjustSize();
            qrMess = qlMess->geometry();
            qrMess.setWidth( qrMess.width() * 1.5 );
            qrMess.setHeight( qrMess.height() * 1.5 );
            qlMess->setGeometry( qrMess );

//            resize( qrMess.size() );
            adjustSize();
        }
        else
            qlMess->setVisible ( false );
    }
}

// ******************* Protected Q_SLOTS ******************************


/* bool PanelsWindow::event(QEvent *event)
{
//    qDebug() << "\033[91m" << "qDebug(): PanelWindow::event event->type()" << event->type() << "\033[0m";
  
    switch (event->type())
    {
//    case QEvent::NonClientAreaMouseButtonRelease:
    case QEvent::Resize:
//        if (event->spontaneous()) return false;   
    case QEvent::UpdateRequest:
        qtResize.stop();
        qtResize.start(500);

        break;          
    default:
      
      break;
    }
  
  return QWidget::event (event);
}

void PanelsWindow::resizeDone()
{
    QSize qsResize, qsSnap, qsObj;
    QRect qrGeom;
  
    qsResize = size();    
    qsSnap.setWidth(int((qsResize.width() / 64.0) + 0.5) * 64.0);
    qsSnap.setHeight(int((qsResize.height() / 64.0) + 0.5) * 64.0);

//    if (qsSnap != qsResize)
    {
      resize(qsResize.width(), qsResize.height() -1);
      resize(qsSnap);
    }
} */


/***************************************************************************
 * Private methods                                                         *
 ***************************************************************************/

void PanelsWindow::childEvent(QChildEvent *e)
{
    if (e->added()) 
    {
        Panel *panel = qobject_cast<Panel *>( e->child() );
      
        loadDisplayConfig( /* (Panel *)e->child() */ panel );
    }
}

// ************* Draging panels and show menu ************************

void PanelsWindow::childEventReceived(QEvent *e)
{
QMouseEvent *m = (QMouseEvent*)e;

 switch(e->type()) {
  case QEvent::MouseButtonPress:

      if( m->button()==Qt::LeftButton ) startDragChild(m,(QWidget *)sender());
      break;

  case QEvent::MouseButtonRelease:

      if( m->button()==Qt::RightButton ) Q_EMIT rightMouseClicked(m);
      break;
  default:
    
    break;
 }
}


void PanelsWindow::startDragChild(QMouseEvent *m, QWidget *w)
{
	if(childDraging == nullptr )
	{
		childDraging= w;    
		grabMouse(Qt::SizeAllCursor);
#if (QT_VERSION < QT_VERSION_CHECK(6, 0, 0))
		origMousePt = w->mapFromGlobal(m->globalPos());
#else
                origMousePt = w->mapFromGlobal(m->globalPosition().toPoint());
#endif
		//w->move( m->globalPos()-mapToGlobal(QPoint(0,0))-origMousePt);
		w->raise();
		w->update();
	}
}

void PanelsWindow::endDragChild()
{
    int meterX, meterY;

int x= childDraging->x();
int y= childDraging->y();

 if(x<0) x=0;
 if(y<0) y=0;
 releaseMouse();

/*  meterWitdhSize = gaugeSize;
    meterHeightSize = gaugeSize;

    meterX = ((x+childDraging->width() /2) / meterWitdhSize) * meterWitdhSize;
    meterY = ((y+childDraging->height()/2) / meterHeightSize) * meterHeightSize; */

    meterX = ( ( x + meterWidth / 2 ) / meterWidth ) * meterWidth;
    meterY = ( ( y + meterHeight / 2 ) / meterHeight ) * meterHeight;

qDebug() << "\033[91m" << "qDebug(): PanelWindow::endDragChild() " << meterX << meterY << "\033[0m";

 childDraging->move( meterX, meterY );
 
 saveDisplayConfig((Panel *)childDraging);
 childDraging= nullptr;
}

void PanelsWindow::mouseMoveEvent ( QMouseEvent *m )
{
	if(childDraging) 
	{ 
		childDraging->move(m->pos()-origMousePt);
		childDraging->update(); 
	}
}


void PanelsWindow::mouseReleaseEvent ( QMouseEvent *m )
{
 if(childDraging && m->button()==Qt::LeftButton)  endDragChild();
 else
   if(m->button()==Qt::RightButton) Q_EMIT rightMouseClicked(m);
}

// ********************** Panels creation ************************

void PanelsWindow::findUnusedPosition( Panel *displayIn, int *px,int *py )
{
bool found;
uint mx,my,x,y,a[32];

  const QObjectList *list= &children();

 for(y=0; y<32; y++) a[y]= 0;
 mx= 0; my= 0;
 if(list)
 {
    for ( QObjectList::ConstIterator it = list->begin(); it != list->end(); ++it )       
    {
       Panel *display = qobject_cast<Panel *>( *it );
       
       if ( display && display != displayIn )
       {

       y=display->y() / meterHeight;
       x=display->x() / meterWidth;

//       qDebug() << "\033[91m" << "qDebug(): PanelsWindow::findUnusedPosition geometry" << display->geometry() << "y x" << y << x << "\033[0m";

       if(x<32 && y<32) a[y]|=  (1 << x);
       if(x>mx) mx= x;
       if(y>my) my= y;
       
       }
/*       else
         break; */
      
    }
 
 }
 //
 for(x=0,y=0,found=false;y<=my && !found;) {
   found= ((a[y] & (1 << x))==0);
   if(!found)
     if(++x>mx) { x= 0; y++; }
 }

 if(!found) {
    x= (width()-1) / meterWidth;
    if(x>mx) { x= mx+1; y= 0;    }
    else     { x= 0;    y= my+1; }
 }
 //
 *px= x;
 *py= y;
}

void PanelsWindow::loadDisplayConfig(Panel *display)
{
 if( display == nullptr ) return;

    updateMessLabel( false );
 
 loadPalette(display);

 bool fNoCfg;
 QPoint p;
 
    Sensor *sensor = ksensorsMain->sensorDevsList->getSensor( display->objectName() );
//  setGroupConf ( display->objectName() );
    setSensorDevGroupConf ( sensor->getShortName(), sensor->getLongName(), sensor->getSubName() );
    p = getEntryConf ( qsFL1( "showPos" ), meterTypes, QPoint(), ksensorsSetting::AutoDefault );
 
    
// qDebug() << "\033[91m" << "qDebug(): PanelsWindow::loadDisplayConfig geometry" << display->geometry() << "showPos" << p << "\033[0m";
    
 fNoCfg= (p.x()<0 || p.y()<0);
 if(fNoCfg) findUnusedPosition( display, &p.rx(),&p.ry() );

    display->move(p.x()*meterWidth,p.y()*meterHeight);
//    display->move(p.x() * meterWitdhSize, p.y() * meterHeightSize);

  display->resizePanel ( meterWidth, meterHeight );
 
 display->show();
 if(fNoCfg) {
    saveDisplayConfig(display);
    adjustSize();
 }
 display->installEventFilter(this);
 connect(display, SIGNAL(eventReceived(QEvent *)), this, SLOT(childEventReceived(QEvent *)) );
}

void PanelsWindow::saveDisplayConfig(Panel *display)
{
 setGroupConf ( display->objectName() );
// setEntryConf ( qsFL1( "show" ), display->getShowType() );

//    QPoint p= display->pos()/gaugeSize;
    QPoint p = QPoint ( display->pos().x() / meterWidth, display->pos().y() / meterHeight ) ;
 
 setEntryConf ( qsFL1( "showPos" ), meterTypes, p);
}


void PanelsWindow::createSensorPanels()
{
    bool withMeter;

    QString qsSensorShowType;

    MeterTypes sensorMeterTypes;

    withMeter = false;

 const QObjectList *chips = ksensorsMain->sensorDevsList->getSensorsChips();
 if(chips)
 {
    for ( QObjectList::ConstIterator it = chips->begin(); it != chips->end(); ++it )       
    {
      SensorDev *chip = (SensorDev *)(*it);
       
      if ( chip )
      {
        
       QObjectList *list= (QObjectList *)chip->getSensors();
       if(list) 
       {
          for ( QObjectList::ConstIterator it = list->begin(); it != list->end(); ++it )       
          {
            LMSensor *sensor = (LMSensor *)(*it);
       
            if ( sensor )
            {
//              setGroupConf ( sensor->objectName() );
                setSensorDevGroupConf ( sensor->getShortName(), sensor->getLongName(), sensor->getSubName() );
                qsSensorShowType = getEntryConf ( qsFL1( "show" ), qsFL1( "false" ), ksensorsSetting::AutoDefault );

                sensorMeterTypes = MeterTypes::None;
                if ( qsSensorShowType.indexOf ( qsFL1( "gauge" ) ) >= 0 )
                    sensorMeterTypes = (MeterTypes)( sensorMeterTypes | MeterTypes::Gauge );
                if ( qsSensorShowType.indexOf ( qsFL1( "chart" ) ) >= 0 )
                    sensorMeterTypes = (MeterTypes)( sensorMeterTypes | MeterTypes::Chart );

                if ( sensorMeterTypes & meterTypes )
                {
                    withMeter = true;

                    (void) new SensorPanel( this, sensor, sensor->getLongName() );
                }
            }
            else
              break;
            
           }
       }
      }
      else
        break;
      
    }
    
 }

/* warning, a generic panel window is not managed,
 * if necessary it is necessary to review the whole flow of creating the panels window, panels, deleting panels, readd panels etc. */
    if ( withMeter )
        meterTypes = (MeterTypes)( meterTypes & ~MeterTypes::Empty );
    else
        meterTypes = (MeterTypes)( meterTypes | MeterTypes::Empty );

    updateMessLabel ( true );
}

void PanelsWindow::saveGeometry()
{
  QRect geometry ( pos(), size() );
  
  if ( !(geometry.x() >= 0 && geometry.y() >= 0) )
  {
    geometry.setX( 0 );
    geometry.setY( 0 );
  }

  if ( meterTypes != MeterTypes::None )
  {
  setGroupConf ( qsFL1( "ShowWidget" )  );
  setEntryConf ( qsFL1( "geometry" ), meterTypes, geometry );
  }

//  void PanelWindow::cfgWriteGeometry( QRect geometry )

}

void PanelsWindow::loadGeneralOptions( bool enabForce )
{
  bool newWindowsSolid;
  double windowsOpacityValue;
  
  setGroupConf ( qsFL1( "ShowWidget" )  );
  newWindowsSolid = getEntryConf ( qsFL1( "windowsSolid" ) , false, ksensorsSetting::AutoDefault );
  windowsOpacityValue = getEntryConf ( qsFL1( "windowsOpacityValue" ) , -1.0, ksensorsSetting::AutoDefault );

  if ( enabForce || newWindowsSolid != windowsSolid )
  {
    windowsSolid = newWindowsSolid;

    setAttribute( Qt::WA_TranslucentBackground, !windowsSolid );
    setAttribute( Qt::WA_NoSystemBackground, !windowsSolid );
    setWindowFlags( ( windowsSolid ? ( Qt::WindowStaysOnTopHint | Qt::WindowTitleHint ) : ( Qt::WindowStaysOnTopHint | Qt::FramelessWindowHint | Qt::Tool ) ) );

//    setVisible( true );
    show();
  }
  setWindowOpacity( ( windowsOpacityValue / 100.0 ) );


  updatePanels ( enabForce );
}

// ************************+******** Palettes stuff ****************************

void PanelsWindow::loadPalette(Panel *panel)
{
  QPalette pal = qpPanelWindow;
  if(panel) 
  {
    if(getPalette(pal, panel->objectName() ,false)) 
      panel->setPalette(pal);
    else  
      panel->setPalette( qpPanelWindow /* QPalette() */ );
    
    panel->updateFrame();
  } 
  else 
  {
    getPalette(pal, QString(), true);
    qpPanelWindow = pal;
    setPalette(pal);
    pal.setBrush( backgroundRole(), QBrush( qpmBackGround ));    
    setPalette(pal);
  }
}

/* void PanelWindow::setBackgroundPixmap( const QPixmap &qpmBackGround )
{  
  QPalette pal = palette();
  pal.setBrush( backgroundRole(), QBrush( qpmBackGround ));
  setPalette(pal);
} */
